//------------------------------------------------------------------------------
// File: AsyncFlt.cpp
//
// Desc: DirectShow sample code - implementation of CAsyncFilter.
//
// Copyright (c) 1996-2001 Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------


#include <streams.h>
#include <asyncio.h>
#include <asyncrdr.h>
#include <initguid.h>   // Make our GUID get defined

#pragma warning(disable:4710)  // 'function' not inlined (optimization)
#include "xcdsrc.h"


//
// Setup data for filter registration
//
const AMOVIESETUP_MEDIATYPE sudOpPinTypes =
{ &MEDIATYPE_Stream     // clsMajorType
, &MEDIASUBTYPE_NULL }; // clsMinorType

const AMOVIESETUP_PIN sudOpPin =
{ L"Output"          // strName
, FALSE              // bRendered
, TRUE               // bOutput
, FALSE              // bZero
, FALSE              // bMany
, &CLSID_NULL        // clsConnectsToFilter
, L"Input"           // strConnectsToPin
, 1                  // nTypes
, &sudOpPinTypes };  // lpTypes

const AMOVIESETUP_FILTER sudXcd =
{ &CLSID_RIFFCDXAReader           // clsID
, L"RIFF/CDXA Source"             // strName
, MERIT_UNLIKELY                  // dwMerit
, 1                               // nPins
, &sudOpPin };                    // lpPin


//
//  Object creation template
//
CFactoryTemplate g_Templates[1] = {
    { L"RIFF/CDXA Source"
    , &CLSID_RIFFCDXAReader
    , CXcdFilter::CreateInstance
    , NULL
    , &sudXcd }
};

int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);


//
// Filter registration functions
//
HRESULT DllRegisterServer(){
    return AMovieDllRegisterServer2(TRUE);
}


HRESULT DllUnregisterServer(){
    return AMovieDllRegisterServer2(FALSE);
}


/* Create a new instance of this class */
CUnknown *CXcdFilter::CreateInstance(LPUNKNOWN pUnk, HRESULT *phr){
    /*  DLLEntry does the right thing with the return code and
        returned value on failure
    */
    return new CXcdFilter(pUnk, phr);
}


//////////////////////////////////////////////////////////////////////////////
//
// CMemStream methods
//


CMemStream::CMemStream() {
	dprintf ("default empty CTOR for CMemStream\n");
}


    /*  Initialization */
void CMemStream::Init(XCD_Media_Reader* xcdReader, DWORD dwKBPerSec){
    dprintf ("CMemStream::Init");
	m_xcdReader=xcdReader;
	m_strippedLength=m_xcdReader->size();
    m_dwKBPerSec = dwKBPerSec;
    m_dwTimeStart = timeGetTime();
	dprintf ("strippedLength=%d",(long)m_strippedLength);
    dprintf ("CMemStream::Init: end");
}


HRESULT CMemStream::SetPointer(LONGLONG llPos){
	dprintf ("CMemStream::SetPointer(%d)",llPos);

    if (llPos < 0 || llPos > m_strippedLength) {
		dprintf ("CMemStream::SetPointer returned: false");
        return S_FALSE;
    } else {
        m_strippedPosition = (long)llPos;
		dprintf ("CMemStream::SetPointer returned: true");
        return S_OK;
    }
}


HRESULT CMemStream::Read(PBYTE pbBuffer, DWORD dwBytesToRead, BOOL bAlign, LPDWORD pdwBytesRead){
    dprintf ("CMemStream::Read2\n");
	CAutoLock lck(&m_csLock);
    DWORD dwReadLength;

    /*  Wait until the bytes are here! */
    DWORD dwTime = timeGetTime();

    if ((long)(m_strippedPosition + dwBytesToRead) > m_strippedLength) {
        dwReadLength = (DWORD)(m_strippedLength - m_strippedPosition);
    } else {
        dwReadLength = dwBytesToRead;
    }
    DWORD dwTimeToArrive =
        (m_strippedPosition + dwReadLength) / m_dwKBPerSec;

    if (dwTime - m_dwTimeStart < dwTimeToArrive) {
        Sleep(dwTimeToArrive - dwTime + m_dwTimeStart);
    }

//    strippedReadFile((PVOID)pbBuffer, m_strippedPosition, dwReadLength);
    m_xcdReader->read((PVOID)pbBuffer, m_strippedPosition, dwReadLength);

	m_strippedPosition+=dwReadLength;
    *pdwBytesRead = (DWORD)dwReadLength;
    dprintf ("CMemStream::Read (media) %u bytes @%d  end\n",dwReadLength, m_strippedPosition-dwReadLength);
    return S_OK;
}


LONGLONG CMemStream::Size(LONGLONG *pSizeAvailable){
	dprintf ("CMemStream::Size\n");
    LONGLONG llCurrentAvailable =
        static_cast <LONGLONG> (UInt32x32To64((timeGetTime() - m_dwTimeStart),m_dwKBPerSec));

	*pSizeAvailable =  min((LONGLONG)m_strippedLength, llCurrentAvailable);
	dprintf ("CMemStream::Size(=available=%I64d)=%d",*pSizeAvailable, m_strippedLength);
	dprintf ("CMemStream::Size end\n");
	return m_strippedLength;
}


DWORD CMemStream::Alignment(){
    return 1;
}


void CMemStream::Lock(){
    m_csLock.Lock();
}


void CMemStream::Unlock(){
    m_csLock.Unlock();
}

/*
long CMemStream::strippedReadFile(void* dest, long strippedPos, long length){
	dprintf ("CMemStream::strippedReadFile\n");
	long res=m_xcdReader->read(dest, strippedPos, length);
	dprintf ("CMemStream::strippedReadFile and\n");
	return res;
}
*/

//////////////////////////////////////////////////////////////////////////////
//
// CXcdFilter methods
//

CXcdFilter::CXcdFilter(LPUNKNOWN pUnk, HRESULT *phr) :
        CAsyncReader(NAME("Mem Reader"), pUnk, &m_Stream, phr),
			m_pFileName(NULL), m_xcdReader(NULL){

	dprintf ("CxcdFilter::CTOR");
}


CXcdFilter::~CXcdFilter(){
	if (m_pFileName)
		delete [] m_pFileName;

	if (m_xcdReader)
		delete m_xcdReader;

	dprintf ("CxcdFilter::DTOR");
}


STDMETHODIMP CXcdFilter::NonDelegatingQueryInterface(REFIID riid, void **ppv){
    if (riid == IID_IFileSourceFilter) {
        return GetInterface((IFileSourceFilter *)this, ppv);
    } else {
        return CAsyncReader::NonDelegatingQueryInterface(riid, ppv);
    }
}

/*  IFileSourceFilter methods */

//  Load a (new) file
STDMETHODIMP CXcdFilter::Load(LPCOLESTR lpwszFileName, const AM_MEDIA_TYPE *pmt){

    dprintf ("CXcdFilter::Load\n");
	CheckPointer(lpwszFileName, E_POINTER);

    // lstrlenW is one of the few Unicode functions that works on win95
    int cch = lstrlenW(lpwszFileName) + 1;

#ifndef UNICODE
    TCHAR *lpszFileName=0;
    lpszFileName = new char[cch * 2];
    if (!lpszFileName) {
	    dprintf ("CXcdFilter::Load fail:E_OUTOFMEMORY\n");
      	return E_OUTOFMEMORY;
    }
    WideCharToMultiByte(GetACP(), 0, lpwszFileName, -1,
    		lpszFileName, cch, NULL, NULL);
#else
    TCHAR lpszFileName[MAX_PATH]={0};
    lstrcpy(lpszFileName, lpwszFileName);
#endif
    CAutoLock lck(&m_csFilter);

    /*  Check the file type */
    CMediaType cmt;
    if (NULL == pmt) {
        cmt.SetType(&MEDIATYPE_Stream);
        cmt.SetSubtype(&MEDIASUBTYPE_NULL);
		dprintf ("pmt=null\n");
    } else if (pmt->subtype == MEDIASUBTYPE_MPEG1VideoCD ) {
        cmt.SetType(&MEDIATYPE_Stream);
        cmt.SetSubtype(&MEDIASUBTYPE_NULL);
		dprintf (" pmt->subtype == MEDIASUBTYPE_MPEG1VideoCD\n");
    } else {
		dprintf ("pmt not null and not riff\n");
        cmt = *pmt;
    }


	if (!OpenMedia(lpszFileName)){
#ifndef UNICODE
        delete [] lpszFileName;
#endif
	    dprintf ("CXcdFilter::Load fail:OpenTheFile\n");
		return E_FAIL;
    }


	dprintf("calling m_stream.init\n");
    m_Stream.Init(m_xcdReader);

///// Detect Media Type //////

	dprintf("detecting Media Type\n");
	PBYTE pbData = new BYTE[SECTOR_USER_DATA];
	m_xcdReader->read(pbData, 0, SECTOR_USER_DATA);
	DetectMediaType(pbData);
	delete [] pbData;
    
	cmt.SetType(&MediaTypeGUID);
	cmt.SetSubtype(&MediaSubtypeGUID);

//////////////////////////////
	
	m_pFileName = new WCHAR[cch];

    if (m_pFileName!=NULL)
    	CopyMemory(m_pFileName, lpwszFileName, cch*sizeof(WCHAR));

    // this is not a simple assignment... pointers and format
    // block (if any) are intelligently copied
    m_mt = cmt;

    /*  Work out file type */
    cmt.bTemporalCompression = TRUE;	       //???
    cmt.lSampleSize = 1;
    dprintf ("CXcdFilter::Load end\n");

    return S_OK;
}


    // Modeled on IPersistFile::Load
    // Caller needs to CoTaskMemFree or equivalent.

STDMETHODIMP CXcdFilter::GetCurFile(LPOLESTR * ppszFileName, AM_MEDIA_TYPE *pmt){
    CheckPointer(ppszFileName, E_POINTER);
    *ppszFileName = NULL;

	dprintf ("CXcdFilter::GetCurFile\n");

    if (m_pFileName!=NULL) {
        DWORD n = sizeof(WCHAR)*(1+lstrlenW(m_pFileName));

        *ppszFileName = (LPOLESTR) CoTaskMemAlloc( n );
        if (*ppszFileName!=NULL) {
              CopyMemory(*ppszFileName, m_pFileName, n);
        }
    }

    if (pmt!=NULL) {
        CopyMediaType(pmt, &m_mt);
    }
	dprintf ("CXcdFilter::GetCurFile end\n");
    return NOERROR;
}


/////////////////////////////////////////////////////////////////////////////
//
// File reading stuff
//

BOOL CXcdFilter::OpenMedia(LPCTSTR lpszFileName){

	dprintf ("CXcdFilter::OpenMedia('%s')\n",lpszFileName);
	m_xcdReader=new XCD_Media_Reader(lpszFileName);
	if (!(m_xcdReader->isOpenedOK())){
		dprintf ("Could not initialize the XCD reader\n");
		return FALSE;
	}

	dprintf ("CXcdFilter::OpenMedia end\n");
    return TRUE;
}


int CopyToken (char *dest, char *szValue, int *i){
	
	int d = 0;
	while(*i < (int)strlen(szValue))
	{
		if (szValue[*i] == ' ') { *i += 1; continue; }
		if (szValue[*i] == ',') { *i += 1; break; }
		dest[d++] = szValue[*i];
		*i += 1;
	}
	dest[d] = '\0';

	if (*i == (int)strlen(szValue)) return 1; // end of string

	return 0;
}


BOOL CXcdFilter::DetectMediaType(BYTE *Data){

	#define BUFSIZE MAX_PATH+1

	HKEY hkHKCR_MediaTypeRoot;
	HKEY hkMediaType;
	HKEY hkMediaSubtype;
	CHAR szValueIndex[2];
	TCHAR szValue[BUFSIZE];
	DWORD dwValueSize;

	CHAR szMediaSubtypeName[MAX_PATH] = {0};

	BOOL Found = FALSE;

	
	LPOLESTR wszMediaType;
	CHAR szMediaTypeName[MAX_PATH];
	StringFromCLSID(MEDIATYPE_Stream, &wszMediaType);
	if (WideCharToMultiByte(CP_ACP, 0, wszMediaType, -1, szMediaTypeName, MAX_PATH, NULL, NULL) == 0)
	CoTaskMemFree(&wszMediaType);

	dprintf("MediaType Key: %s\n",szMediaTypeName);

	RegOpenKeyEx(HKEY_CLASSES_ROOT, "Media Type", NULL, KEY_ALL_ACCESS, &hkHKCR_MediaTypeRoot);
	RegOpenKeyEx(hkHKCR_MediaTypeRoot, szMediaTypeName, NULL, KEY_ALL_ACCESS, &hkMediaType);

	dprintf("MediaSubtype Keys:\n");

	int i = 0;
	while(!Found && RegEnumKey(hkMediaType, i++, szMediaSubtypeName, sizeof(szMediaSubtypeName)) == ERROR_SUCCESS)
	{
		RegOpenKeyEx(hkMediaType, szMediaSubtypeName, NULL, KEY_ALL_ACCESS, &hkMediaSubtype);
		
		dprintf ("%s\n",szMediaSubtypeName);

		int j = 0;
		while (!Found &&
			   j < 10 &&
			   (dwValueSize = BUFSIZE,
			   RegQueryValueEx(hkMediaSubtype, _itoa(j++, szValueIndex, 10), NULL, NULL, (LPBYTE)szValue, &dwValueSize) == ERROR_SUCCESS))
		{
			// parse szValue and compare with Data
			// once found, set Found = TRUE

			dprintf("\"%s\"=\"%s\"\n",szValueIndex,szValue);
			
			LONG offset, length;
			char mask[MAX_PATH], content[MAX_PATH], tmpstr[MAX_PATH];

			int m = 0;
			int is_end = 0;
			int failed = 0;

			while(!is_end && !failed)
			{

				CopyToken(tmpstr, szValue, &m);
				if (strlen(tmpstr) == 0) { failed = 1; continue; }
				offset = atoi(tmpstr);
				CopyToken(tmpstr, szValue, &m);
				if (strlen(tmpstr) == 0) { failed = 1; continue; }
				length = atoi(tmpstr);
				CopyToken(mask, szValue, &m);
				is_end = CopyToken(content, szValue, &m);
				if (strlen(tmpstr) == 0) { failed = 1; continue; }
				

//				dprintf("data: %d,%d,\"%s\",\"%s\"", offset, length, mask, content);

				int l = 0;

//				for (l = 0; l < length; l++)
//					dprintf("%x",Data[l+offset]);

				char tmp_str[3] = { 0, };
				BYTE Data_byte, content_byte;
				for (l = 0; l < length && !failed; l++)
				{

					strncpy(tmp_str, &(content[l*2]), 2);
					content_byte = (BYTE)strtoul(tmp_str, NULL, 16);

					Data_byte = Data[l+offset];

					if ((LONG)strlen(mask) == length*2) // valid mask string
					{
						strncpy(tmp_str, &(mask[l*2]), 2);
						Data_byte &= (BYTE)strtoul(tmp_str, NULL, 16);
						content_byte &= (BYTE)strtoul(tmp_str, NULL, 16);
					}

					if (Data_byte != content_byte)
						failed = 1;
					else
						dprintf("match: %x (%c)", Data[l+offset], Data[l+offset]);

				}
			}

			if (!failed) Found = TRUE; // FOUND!!!

		}

		RegCloseKey(hkMediaSubtype);
	}

	RegCloseKey(hkMediaType);
	RegCloseKey(hkHKCR_MediaTypeRoot);
	
	if (Found)
	{
		MediaTypeGUID = MEDIATYPE_Stream;

		WCHAR wszSubtypeName[MAX_PATH];
		MultiByteToWideChar(CP_ACP, 0, szMediaSubtypeName, -1, wszSubtypeName, MAX_PATH);
		CLSIDFromString(wszSubtypeName, &MediaSubtypeGUID);
	}
	else
	{
		MediaTypeGUID = MEDIATYPE_Stream;
		MediaSubtypeGUID = MEDIASUBTYPE_NULL;
	}
	
	if (Found)
		dprintf("Found!");
	else
		dprintf("Not found... using NULL subtype");

	return Found;
}
